// Chunk.h: Contains everything related to chunk.
//          Structure declaration                
//          List of Chunk ID     
//          ...              
//
//////////////////////////////////////////////////////////////////////

#ifndef _CHUNK_H__
#define _CHUNK_H__

#include <stdio.h>
#include <vector>

struct stChunk{		unsigned short type;
			unsigned int   size;
			unsigned int   read; };

struct stChunkToSave{	unsigned short type;						
			unsigned int   size;
			void*		   data; 
			void*		   buffer; 
			std::vector<stChunkToSave*>	daughters; };


#define	C_PRIMARY					55555	// Primary ChunkId
											// Other Chunks are daughters of 
				 							// this PrimaryChunk 


#define	C_FEB_DETID					1000	// Contains a FROG_ELEMENT_BASE_WITH_DETID
#define	C_FEB_DETID_COMPACT			2000	// Contains a FROG_ELEMENT_BASE_WITH_DETID

#define	C_FEB_DETID_NAME			 3000	// Contains a FROG_ELEMENT_BASE_WITH_DETID_AND_NAME




#define	C_EVENT						10000		// Contains Chunks related to the Event
												// [10000,19999] is reserved for Event data
#define	C_EVENTR					10001		// Contains Chunks related to the Event with RunNumber
#define	C_EVENTRT					10002		// Contains Chunks related to the Event with RunNumber and TimeStamp

#define	C_SIM						11000		// Contains Chunks related to Simulation
												// [11000,11999] is reserved for Simulation data
#define	C_SIM_HIT					11010		// Contains PSimHits
#define	C_SIM_VERTEX				11020		// Contains SimVertex
#define	C_SIM_TRACK					11030		// Contains SimTracks

#define	C_RECO						12000		// Contains Chunks related to Reconstruction
												// [12000,12999] is reserved for Reconstruction data
#define	C_RECO_TRACK				12110		// Contains Chunks related to RecoTracks
#define	C_RECO_HIT   				12111		// Contains RecHits
#define	C_RECO_SEGMENT 				12112		// Contains Segments
#define	C_RECO_CLUSTER_SISTRIP		12113		// Contains SiStrip RecHits
#define	C_RECO_CLUSTER_PIXEL		12114		// Contains Pixel   RecHits

#define C_RECO_CALOHIT				12210		// Contains Chunks related to Ecal/Hcal Clusters/Hits
#define C_RECO_CALOTOWER			12310		// Contains Chunks related to CaloTowers

#define C_RECO_MUONSEG				12410		// Contains Chunks related to Muon Segment
#define C_RECO_MUONHIT				12420		// Contains Chunks related to Muon Hits

#define C_RECO_NUCL					12500		// Contains Chunks related to Nuclear interaction

#define C_RECO_JET					13100		// Contains Chunks related to Jet

#define C_RECO_MET					14100		// Contains Chunks related to Jet

#define C_RECO_CANDIDATE        	15100       // Contains Chunks related to RecoCandidate

#define	C_GEOMETRY					20000	// Contains Chunks related to the Geometry
											// [20000,29999] is reserved for geometry


#define	C_TRACKER					21000	// Contains Chunks related to the Tracker Geometry
											// [21000,21999] is reserved for the Tracker Geometry
#define	C_TRACKER_MOD				21100   // Contains data of a Module


#define	C_CALO_MOD					11999   // Contains data of a CaloModule //CHUNCK ID TO FIX

#define	C_ECAL  					22000	// Contains Chunks related to the ECAL Geometry
											// [22000,22999] is reserved for the ECAL Geometry
#define	C_ECAL_MOD					22100   // Contains data of a Module
						


#define	C_HCAL  					23000	// Contains Chunks related to the HCAL Geometry
											// [23000,23999] is reserved for the HCAL Geometry
#define	C_HCAL_MOD					23100   // Contains data of a Module




#define	C_MUON  					24000	// Contains Chunks related to the Muon Geometry
											// [24000,24999] is reserved for the Muon Geometry
#define	C_MUON_MOD					24100   // Contains data of a Module

#define	C_FWD  						25000	// Contains Chunks related to the FWD Geometry
											// [23000,23999] is reserved for the FWD Geometry
#define	C_FWD_MOD					25100   // Contains data of a Module



#define	C_PRIMITIVE					40000	// Contains Chunks related to Primtive (very basic shape)
											// [40000,49999] is reserved for primitive

#define	C_PRIMITIVE_SOLID			41000	// Contains Chunks related to Solid Primtive (Cube, Sphere, ...)
											// [41000,41999] is reserved for primitive

#define	C_PRIMITIVE_CUSTOM_CUBE		41010   // Contains a custom solid defined by 8 corners
#define	C_PRIMITIVE_CUBE			41020   // Contains a cube
#define	C_PRIMITIVE_SPHERE			41030   // Contains a sphere
#define	C_PRIMITIVE_PARTIALSPHERE	41031   // Contains a sphere
#define	C_PRIMITIVE_CYLINDER		41040	// Contains a cylinder

#define	C_PRIMITIVE_SURFACE			42000	// Contains Chunks related to Surface Primtive (Plane, Disc, ...)
											// [42000,42999] is reserved for primitive

#define	C_PRIMITIVE_CUSTOM_SURFACE	42010   // Contains a custom surface defined by 4 corners
#define	C_PRIMITIVE_RECTANGLE		42020   // Contains a rectangle
#define	C_PRIMITIVE_DISC			42030   // Contains a disc

#define	C_PRIMITIVE_LINES			43000	// Contains Chunks related to Lines Primtive (Line Segment, Circle, ...)
#define	C_PRIMITIVE_POINT			43010	// Contains Chunks related to Lines Primtive (Line Segment, Circle, ...)
#define	C_PRIMITIVE_LINE			43020	// Contains Chunks related to Lines Primtive (Line Segment, Circle, ...)
#define	C_PRIMITIVE_CIRCLE			43030	// Contains Chunks related to Lines Primtive (Line Segment, Circle, ...)

void ReadChunk (FILE* pFile, stChunk* pChunk);
void WriteChunk(FILE* pFile, stChunkToSave* pChunk);


#endif
