// FROG_Element_Base_With_DetId.h: base class for all modules (which contains a method DetId()

// WARNING : Class heriting from FROG_Element_Base_With_DetId are responsible for saving/loading detId_
//           Loading and writting function of FROG_Element_Base_With_DetId are usefull only to make
//           a group of daughters (branchs) with a detId.


//////////////////////////////////////////////////////////////////////

#ifndef _FROG_ELEMENT_BASE_WITH_DETID_H__
#define _FROG_ELEMENT_BASE_WITH_DETID_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects.h"
#include "FROG_DetId.h"

class FROG_Element_Base_With_DetId : public FROG_Element_Base {
public : 
	typedef std::map<unsigned int,FROG_Element_Base*,std::less<unsigned int> >   		 Frog_map;

protected :
	unsigned int detId_;
	FROG_Objects*  frogObjects_;
	Frog_map       DetId_map;

public :
	virtual bool        isCompactible(){ return false; }
	static unsigned int sizeOf()      { return 6 + sizeof(unsigned int);}

	FROG_Element_Base_With_DetId(unsigned short type) : FROG_Element_Base(type), detId_(0){
		size_ = sizeOf();
	}

	FROG_Element_Base_With_DetId(unsigned short type, unsigned int detId) : FROG_Element_Base(type), detId_(detId){
		size_ = sizeOf();
	}

	FROG_Element_Base_With_DetId(unsigned short type, FILE* pFile) : FROG_Element_Base(type){
        size_ = sizeOf();
        fread(&detId_ ,sizeof(detId_),1,pFile);		
    }

    // Fill data_ and size_ with data
    virtual void write(){
		std::vector< std::pair<unsigned int, unsigned int > >* blockOfDaughters = write_init();
		data_ = FillBuffer(data_, &detId_, sizeof(int)); // write the type		  
		write_daughters(blockOfDaughters);
	}

	unsigned int DetId() const { return detId_; }

	
	virtual void printInfos(char* buffer){sprintf(buffer,"DetId = %i", detId_);}


    FROG_Element_Base* getDaughterWithDetId(unsigned int detid) const {
		for(unsigned int i=0;i<daughters_.size();i++){
			if( FROG_Element_Base_With_DetId* FEB_DetId = dynamic_cast<FROG_Element_Base_With_DetId*>(daughters_[i])){
				if(FEB_DetId->DetId()==detid)return FEB_DetId;
			}
		}
		return NULL;
    }
	

	void FillMap(FROG_Element_Base* mother=NULL) {	
		if(mother==NULL){
			DetId_map.clear();
			mother = this;			
		}

		for( unsigned int i=0; i<mother->getDaughterSize(); i++) {	
			if(FROG_Element_Base_With_DetId* mod = dynamic_cast<FROG_Element_Base_With_DetId*>(mother->getDaughter(i))){
				DetId_map[ mod->DetId() ] = mother->getDaughter(i);
				FillMap(mother->getDaughter(i));
			}
		}	
	}


	
    FROG_Element_Base* FindByDetId_FromMap(unsigned int DetId){
		if(DetId==0)return NULL;

		if(DetId_map.size()==0)FillMap();

		Frog_map::iterator it = DetId_map.find(DetId);
		if(it==DetId_map.end())return NULL;		
		return it->second;
		
		return NULL;
    }

	

	virtual void init(void* frogObjects){
        if(display_list_>0) { 
			#ifdef FROG_OPENGL
				glDeleteLists(display_list_, 1); 
				display_list_=0; 
			#endif
		}

		frogObjects_ = (FROG_Objects*) frogObjects;	
		
		if(mother_ != NULL){ 			
			style_ = new FROG_Objects_Style(mother_->style_);
		}else{				 			
			style_ = new FROG_Objects_Style(); 
		}

		frogObjects_->frogCard_->GetColor( style_->color_     ,"Id_%i_Color"		,detId_);
		frogObjects_->frogCard_->GetFloat(&style_->thickness_ ,"Id_%i_Thickness"	,detId_);
		frogObjects_->frogCard_->GetShort(&style_->marker_    ,"Id_%i_Marker"		,detId_);
		frogObjects_->frogCard_->GetShort(&style_->markerSize_,"Id_%i_MarkerSize"	,detId_);
		frogObjects_->frogCard_->GetFloat(&style_->PtMin_     ,"Id_%i_PtMin"		,detId_);
		frogObjects_->frogCard_->GetFloat(&style_->EMin_      ,"Id_%i_EMin"			,detId_);
		style_->ShowDet_ = frogObjects_->frogCard_->GetBool (style_->ShowDet_   ,"Id_%i_ShowDet"		,detId_);

		for(unsigned int i=0;i<daughters_.size();i++)	daughters_[i]->init(frogObjects);
	}

};
#endif

