#ifndef _FROG_ELEMENT_EVENT_CALOTOWER_H__
#define _FROG_ELEMENT_EVENT_CALOTOWER_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects_Extended.h"

class FROG_Element_Event_CaloTower : public FROG_Element_Base_With_DetId {
public :
    float Eem_; float Ehad_; 
	FROG_Objects_Extended*  frogObjects_;
	unsigned int			  Components_CollEvtId_;
	std::vector<unsigned int> Components_Vector_;
	float Em_Color_[4];
	unsigned int nameId_;

    virtual bool isCompactible(){ return false; }

    static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 2*sizeof(float); }

    FROG_Element_Event_CaloTower( unsigned int DetId, float Eem, float Ehad):
    FROG_Element_Base_With_DetId(C_RECO_CALOTOWER, DetId),
    Eem_(Eem),Ehad_(Ehad)
    {
            size_ = sizeOf();
    }

    FROG_Element_Event_CaloTower(FILE* pFile) : FROG_Element_Base_With_DetId(C_RECO_CALOHIT)
    {
            size_ = sizeOf();
            fread(&detId_        ,sizeof(detId_)      ,1,pFile);
            fread(&Eem_          ,sizeof(Eem_)        ,1,pFile);
            fread(&Ehad_         ,sizeof(Ehad_)       ,1,pFile);			
			if(Eem_<0)Eem_*=-1;
    }

    unsigned int ReadComponents(FILE* pFile, unsigned int ToRead)
    {
			if(ToRead==0)return 0;			
			if(ToRead<8)printf("FROG_Element_Event_CaloTower::ReadComponents --> ToRead<8 --> BUG\n");

			fread(&Components_CollEvtId_  ,sizeof(unsigned int)     ,1,pFile);
			unsigned int NComp = (ToRead/sizeof(unsigned int))-1;

			//printf("CompColl = %u\n",Components_CollEvtId_);
			unsigned int TempComp;
			for(unsigned int i=0;i<NComp;i++){
				fread(&TempComp      ,sizeof(unsigned int)     ,1,pFile);
				Components_Vector_.push_back(TempComp);
				//printf("Comp = %i\n",TempComp);
			}

			return (NComp+1)*sizeof(unsigned int);
    }

    virtual void write () {
            size_ = sizeOf();
			if(Components_Vector_.size()>0)size_ += sizeof(unsigned int) + Components_Vector_.size()*sizeof(unsigned int);
            data_ = new unsigned char[size_-6];
            data_ = FillBuffer( data_, &detId_,          sizeof(detId_));
            data_ = FillBuffer( data_, &Eem_,            sizeof(Eem_));
            data_ = FillBuffer( data_, &Ehad_,           sizeof(Ehad_));            

			if(Components_Vector_.size()>0){
				data_ = FillBuffer( data_, &Components_CollEvtId_,              sizeof(Components_CollEvtId_));
				for(unsigned int i=0;i<Components_Vector_.size();i++){
					data_ = FillBuffer( data_, &Components_Vector_[i],          sizeof(Components_Vector_[i]));				
				}
			}

            data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );
    }

	virtual void printInfos(char* buffer){		
		sprintf(buffer,"CaloTower : DetId=%i Eem=%7.2f Ehad=%7.2f",detId_,Eem_,Ehad_);
	}

    #ifdef FROG_OPENGL	
    virtual void display(bool UseDisplayList=true, float* color=NULL){	

		// first Init Colors & Style!
		init(NULL);
		if(Eem_ + Ehad_ <style_->EMin_){return;}

		if(Start_Display(UseDisplayList)){			
			

			if(Components_Vector_.size()==0){ //Display The CaloTowers as a Tower			
				
				FROG_Element_Geom_CaloMod* cell = dynamic_cast<FROG_Element_Geom_CaloMod*>(frogObjects_->frogGeom_->FindByDetId(detId_));
				if(!cell){End_Display(UseDisplayList);printf("TEST\n");return;}
				
				glLoadName( nameId_ );
				if(color){				
					float tempColor[4];
					tempColor[0]  = color[0] + 0.2;
					tempColor[1]  = color[1] + 0.2;
					tempColor[2]  = color[2] + 0.2;
					tempColor[3]  = color[3];

					cell->displayE( 0.75*log10(Eem_/style_->EMin_), tempColor, 0.75*log10(Ehad_/style_->EMin_), color, style_->ShowDet_);
				}else{
					cell->displayE( 0.75*log10(Eem_/style_->EMin_), Em_Color_, 0.75*log10(Ehad_/style_->EMin_), style_->color_, style_->ShowDet_);
				}					
			}else{ //Display the CaloTowers as a set of Cristals Hits
				FROG_Element_Base_With_DetId* Coll = (FROG_Element_Base_With_DetId*)frogObjects_->frogEvent_->FindByDetId(Components_CollEvtId_);
				if(!Coll){End_Display(UseDisplayList);return;}
				
				for(unsigned int i=0;i<Components_Vector_.size();i++){
					FROG_Element_Base* CristalHit = Coll->FindByDetId_FromMap(Components_Vector_[i]);
					if(CristalHit){
						if(color){CristalHit->display(CristalHit->displayList()>0,color);}
						else     {CristalHit->display(CristalHit->displayList()>0,style_->color_);}
					}
				}		
			}

			End_Display(UseDisplayList);
		}
    }
    #endif

	virtual void init(void* frogObjects){
		
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			style_        = mother_->style_;

			Em_Color_[0]  = style_->color_[0] + 0.2;
			Em_Color_[1]  = style_->color_[1] + 0.2;
			Em_Color_[2]  = style_->color_[2] + 0.2;
			Em_Color_[3]  = style_->color_[3];
		}				
	}
};

#endif
