#ifndef _FROG_ELEMENT_EVENT_TRACKHIT_H__
#define _FROG_ELEMENT_EVENT_TRACKHIT_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects_Extended.h"

#ifdef FROG_OPENGL
	#include "../GL/glext.h"
#endif

class FROG_Element_Event_Hit : public FROG_Element_Base_With_DetId {
public :
    float x; float y; float z; float Charge;
	FROG_Objects_Extended*  frogObjects_;
	unsigned int nameId_;

    virtual bool isCompactible(){ return true; }

    static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 4*sizeof(float); }

    FROG_Element_Event_Hit( unsigned int DetId,
    float x_, float y_, float z_, float Charge_):
    FROG_Element_Base_With_DetId(C_RECO_HIT, DetId),
    x(x_),y(y_),z(z_),Charge(Charge_)
    {
            size_ = sizeOf();
    }

    FROG_Element_Event_Hit(FILE* pFile) : FROG_Element_Base_With_DetId(C_RECO_HIT)
    {
            size_ = sizeOf();
            fread(&detId_        ,sizeof(detId_)      ,1,pFile);
            fread(&x             ,sizeof(x)           ,1,pFile);
            fread(&y             ,sizeof(y)           ,1,pFile);
            fread(&z             ,sizeof(z)           ,1,pFile);
            fread(&Charge        ,sizeof(Charge)      ,1,pFile);			
    }

    virtual void write () {
            size_ = sizeOf();
            data_ = new unsigned char[size_-6];
            data_ = FillBuffer( data_, &detId_,          sizeof(detId_));
            data_ = FillBuffer( data_, &x,               sizeof(x));
            data_ = FillBuffer( data_, &y,               sizeof(y));
            data_ = FillBuffer( data_, &z,               sizeof(z));
            data_ = FillBuffer( data_, &Charge,          sizeof(Charge));
            data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );
    }

	virtual void printInfos(char* buffer){		
		sprintf(buffer,"Hit : DetId=%i x=%+7.2f y=%+7.2f z=%+7.2f Charge=%E",detId_, x,y,z,Charge);
	}

    #ifdef FROG_OPENGL	
    virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		if(Start_Display(UseDisplayList)){
			glLoadName( nameId_ );
			FROG_Light::SwitchLight_NoNormal();

			if(style_->marker_>0){
				glBindTexture(GL_TEXTURE_2D,frogObjects_->getMarker(style_->marker_));
				glEnable(GL_TEXTURE_2D);
				glTexEnvf( GL_POINT_SPRITE_ARB, GL_COORD_REPLACE_ARB, GL_TRUE );
				glEnable( GL_POINT_SPRITE_ARB );
			}
		
			glPointSize (style_->markerSize_); 		
			glBegin (GL_POINTS);
			glVertex3f(x, y, z);
			glEnd(); 

			if(style_->marker_>0){
				glDisable( GL_POINT_SPRITE_ARB );
				glDisable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D,0);
			}
			FROG_Light::SwitchLight_Normal();
			
			//if(FROG_Element_Base_With_DetId* det = dynamic_cast<FROG_Element_Base_With_DetId*>(frogObjects_->frogGeom_->FindByDetId(detId_))){
			if(FROG_Element_Geom_TrackerMod* det = dynamic_cast<FROG_Element_Geom_TrackerMod*>(frogObjects_->frogGeom_->FindByDetId(detId_))){

				glBegin (GL_LINES);
				glVertex3f(x+det->LengthX, y+det->LengthY, z+det->LengthZ);
				glVertex3f(x-det->LengthX, y-det->LengthY, z-det->LengthZ);
				glEnd(); 

				det->displayLines();
			}					
			End_Display(UseDisplayList);
		}
    }	
    #endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;			
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = mother_->style_;
			}else{												style_ = new FROG_Objects_Style();	}	
		}		
	}
};

#endif
