#ifndef _FROG_ELEMENT_GEOM_CALOMOD_H__
#define _FROG_ELEMENT_GEOM_CALOMOD_H__

#include "FROG_Element_Base_With_DetId.h"
#include "FROG_Objects.h"
#include "FROG_Primitives.h"

class FROG_Element_Geom_CaloMod : public FROG_Element_Base_With_DetId {
public :
	float ProjFactor;
	float PosX;    float PosY;    float PosZ;					
	float WX;	   float WY;      float WZ;
	float HX;	   float HY;      float HZ;		
	FROG_Objects*  frogObjects_;

	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 10*sizeof(float);}

	FROG_Element_Geom_CaloMod(unsigned int detId, float projFactor,
	float posX,		float posY,		float posZ,
	float wX,		float wY,		float wZ,
	float hX,		float hY,		float hZ, unsigned short chunkType=C_CALO_MOD) :
	FROG_Element_Base_With_DetId(chunkType,detId),
	ProjFactor(projFactor),
	PosX(posX),		PosY(posY),		PosZ(posZ),
	WX(wX),			WY(wY),			WZ(wZ),
	HX(hX),			HY(hY),			HZ(hZ)
	{    
		size_ = sizeOf();		
	}

	FROG_Element_Geom_CaloMod(FILE* pFile, unsigned short chunkType=C_CALO_MOD) : FROG_Element_Base_With_DetId(chunkType)
	{   
		size_ = sizeOf();		
		fread(&detId_		    ,sizeof(detId_)			,1,pFile);
		fread(&ProjFactor	    ,sizeof(ProjFactor)	    ,1,pFile);
		fread(&PosX             ,sizeof(PosX)           ,1,pFile);
		fread(&PosY             ,sizeof(PosY)           ,1,pFile);
		fread(&PosZ             ,sizeof(PosZ)           ,1,pFile);
		fread(&WX				,sizeof(WX)				,1,pFile);
		fread(&WY				,sizeof(WY)				,1,pFile);
		fread(&WZ				,sizeof(WZ)				,1,pFile);
		fread(&HX				,sizeof(HX)				,1,pFile);
		fread(&HY				,sizeof(HY)				,1,pFile);
		fread(&HZ				,sizeof(HZ)				,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		
		data_ = new unsigned char[size_-6];		
		data_ = FillBuffer(		data_, &detId_,			sizeof(detId_));
		data_ = FillBuffer(		data_, &ProjFactor,	    sizeof(ProjFactor));
		data_ = FillBuffer(		data_, &PosX,			sizeof(PosX));
		data_ = FillBuffer(		data_, &PosY,			sizeof(PosY));
		data_ = FillBuffer(		data_, &PosZ,			sizeof(PosZ));
		data_ = FillBuffer(		data_, &WX,				sizeof(WX));
		data_ = FillBuffer(		data_, &WY,				sizeof(WY));
		data_ = FillBuffer(		data_, &WZ,				sizeof(WZ));
		data_ = FillBuffer(		data_, &HX,				sizeof(HX));
		data_ = FillBuffer(		data_, &HY,				sizeof(HY));
		data_ = FillBuffer(		data_, &HZ,				sizeof(HZ));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );						
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){				

		// first Init Colors & Style!
		init(NULL);
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		if(Start_Display(UseDisplayList)){
			displayE(1);
			End_Display(UseDisplayList);
		}

	}

	void displayE(float ERatioA=0, float* colorA=NULL, float ERatioB=0, float* colorB=NULL, bool DrawLine=false){
		// first Init Colors & Style!
		init(NULL);

		if(ERatioA<0)ERatioA=0;
		if(ERatioB<0)ERatioB=0;
		
		float ERatio = ERatioA + ERatioB;
		if(ERatio>1){			
			ERatioA /= ERatio;
			ERatioB /= ERatio;
			ERatio   = 1;
		}
				
		double C1X  =  PosX-WX-HX;				double C1Y  =  PosY-WY-HY;				double C1Z  =  PosZ-WZ-HZ;
		double C2X  =  PosX+WX-HX;				double C2Y  =  PosY+WY-HY;				double C2Z  =  PosZ+WZ-HZ;	
		double C3X  =  PosX+WX+HX;				double C3Y  =  PosY+WY+HY;				double C3Z  =  PosZ+WZ+HZ;
		double C4X  =  PosX-WX+HX;				double C4Y  =  PosY-WY+HY;				double C4Z  =  PosZ-WZ+HZ;	
		double C5XA = (PosX-WX-HX)*ProjFactor;	double C5YA = (PosY-WY-HY)*ProjFactor;	double C5ZA = (PosZ-WZ-HZ)*ProjFactor;
		double C6XA = (PosX+WX-HX)*ProjFactor;	double C6YA = (PosY+WY-HY)*ProjFactor;	double C6ZA = (PosZ+WZ-HZ)*ProjFactor;	
		double C7XA = (PosX+WX+HX)*ProjFactor;	double C7YA = (PosY+WY+HY)*ProjFactor;	double C7ZA = (PosZ+WZ+HZ)*ProjFactor;	
		double C8XA = (PosX-WX+HX)*ProjFactor;	double C8YA = (PosY-WY+HY)*ProjFactor;	double C8ZA = (PosZ-WZ+HZ)*ProjFactor;
		double C5XB = C1X+(C5XA-C1X)*ERatioA;	double C5YB = C1Y+(C5YA-C1Y)*ERatioA;	double C5ZB = C1Z+(C5ZA-C1Z)*ERatioA;
		double C6XB = C2X+(C6XA-C2X)*ERatioA;	double C6YB = C2Y+(C6YA-C2Y)*ERatioA;	double C6ZB = C2Z+(C6ZA-C2Z)*ERatioA;
		double C7XB = C3X+(C7XA-C3X)*ERatioA;	double C7YB = C3Y+(C7YA-C3Y)*ERatioA;	double C7ZB = C3Z+(C7ZA-C3Z)*ERatioA;
		double C8XB = C4X+(C8XA-C4X)*ERatioA;	double C8YB = C4Y+(C8YA-C4Y)*ERatioA;	double C8ZB = C4Z+(C8ZA-C4Z)*ERatioA;

		if(colorA){glColor4fv(colorA);}//else{glColor4fv(style_->color_);}

		DrawCube(C1X ,C1Y ,C1Z ,
				 C2X ,C2Y ,C2Z ,
				 C3X ,C3Y ,C3Z ,
				 C4X ,C4Y ,C4Z ,
				 C5XB,C5YB,C5ZB,
				 C6XB,C6YB,C6ZB,
				 C7XB,C7YB,C7ZB,
				 C8XB,C8YB,C8ZB);

		if(ERatio>ERatioA){			
			double C1XD = C1X+(C5XA-C1X)*ERatio;	double C1YD = C1Y+(C5YA-C1Y)*ERatio;	double C1ZD = C1Z+(C5ZA-C1Z)*ERatio;
			double C2XD = C2X+(C6XA-C2X)*ERatio;	double C2YD = C2Y+(C6YA-C2Y)*ERatio;	double C2ZD = C2Z+(C6ZA-C2Z)*ERatio;
			double C3XD = C3X+(C7XA-C3X)*ERatio;	double C3YD = C3Y+(C7YA-C3Y)*ERatio;	double C3ZD = C3Z+(C7ZA-C3Z)*ERatio;
			double C4XD = C4X+(C8XA-C4X)*ERatio;	double C4YD = C4Y+(C8YA-C4Y)*ERatio;	double C4ZD = C4Z+(C8ZA-C4Z)*ERatio;

			if(colorB){glColor4fv(colorB);}else if(colorA){glColor4fv(colorA);}//else{glColor4fv(style_->color_);}
			DrawCube(C1XD,C1YD,C1ZD,
					 C2XD,C2YD,C2ZD,
					 C3XD,C3YD,C3ZD,
					 C4XD,C4YD,C4ZD,
					 C5XB,C5YB,C5ZB,
					 C6XB,C6YB,C6ZB,
					 C7XB,C7YB,C7ZB,
					 C8XB,C8YB,C8ZB);
		}

		if(DrawLine){
			FROG_Light::SwitchLight_NoNormal();
			if(colorA){glColor4fv(colorA);}//else{glColor4fv(style_->color_);}		
			glLoadName( detId_ );
			glBegin (GL_LINES); 							
				glVertex3f (C1X,C1Y,C1Z);		//+++
				glVertex3f (C4X,C4Y,C4Z);		//+-+
				glVertex3f (C4X,C4Y,C4Z);		//+-+
				glVertex3f (C3X,C3Y,C3Z);		//--+
				glVertex3f (C3X,C3Y,C3Z);		//--+
				glVertex3f (C2X,C2Y,C2Z);		//-++
				glVertex3f (C2X,C2Y,C2Z);		//-++
				glVertex3f (C1X,C1Y,C1Z);		//+++

				glVertex3f (C5XA,C5YA,C5ZA);	//+++
				glVertex3f (C8XA,C8YA,C8ZA);	//+-+
				glVertex3f (C8XA,C8YA,C8ZA);	//+-+
				glVertex3f (C7XA,C7YA,C7ZA);	//--+
				glVertex3f (C7XA,C7YA,C7ZA);	//--+
				glVertex3f (C6XA,C6YA,C6ZA);	//-++
				glVertex3f (C6XA,C6YA,C6ZA);	//-++
				glVertex3f (C5XA,C5YA,C5ZA);	//+++

				glVertex3f (C1X,C1Y,C1Z);		
				glVertex3f (C5XA,C5YA,C5ZA);	

				glVertex3f (C2X,C2Y,C2Z);		
				glVertex3f (C6XA,C6YA,C6ZA);	

				glVertex3f (C3X,C3Y,C3Z);		
				glVertex3f (C7XA,C7YA,C7ZA);	

				glVertex3f (C4X,C4Y,C4Z);		
				glVertex3f (C8XA,C8YA,C8ZA);	
			glEnd(); 
			FROG_Light::SwitchLight_Normal();
		}
	}
	#endif

	
	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = mother_->style_;
			}else{												style_ = new FROG_Objects_Style();	}	
		}
	}


	virtual double getPosX(){	return PosX;	}
	virtual double getPosY(){	return PosY;	}
	virtual double getPosZ(){	return PosZ;	}
	
};
#endif
