#ifndef _FROG_ELEMENT_PRIMITIVE_DISC_H__
#define _FROG_ELEMENT_PRIMITIVE_DISC_H__

#include "FROG_Element_Base.h"

class FROG_Element_Primitive_Disc : public FROG_Element_Base_With_DetId {
public :
	float Radius;
	float PosX;    float PosY;  float PosZ;					
	float DirX;	   float DirY;	float DirZ;	
	unsigned short NPhi;
	FROG_Objects*  frogObjects_;

	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base_With_DetId::sizeOf() + 7*sizeof(float) + sizeof(unsigned short); }

	FROG_Element_Primitive_Disc(
	unsigned int detId,			  float radius,
	float posX,    float posY,    float posZ,
	float dirX,	   float dirY,    float dirZ,
	unsigned short nPhi = 10):
	FROG_Element_Base_With_DetId(C_PRIMITIVE_DISC,detId),
	Radius(radius),
	PosX(posX),			PosY(posY),			PosZ(posZ),
	DirX(dirX),			DirY(dirY),			DirZ(dirZ),
	NPhi(nPhi)
	{    
		size_ = sizeOf();
	}

	FROG_Element_Primitive_Disc(FILE* pFile) : FROG_Element_Base_With_DetId(C_PRIMITIVE_DISC)
	{   
		size_ = sizeOf();
		fread(&detId_           ,sizeof(detId_)			,1,pFile);				
		fread(&Radius           ,sizeof(Radius)			,1,pFile);		
		fread(&PosX             ,sizeof(PosX)           ,1,pFile);
		fread(&PosY             ,sizeof(PosY)           ,1,pFile);
		fread(&PosZ             ,sizeof(PosZ)           ,1,pFile);
		fread(&DirX				,sizeof(DirX)			,1,pFile);
		fread(&DirY				,sizeof(DirY)			,1,pFile);
		fread(&DirZ				,sizeof(DirZ)	        ,1,pFile);
		fread(&NPhi				,sizeof(NPhi)	        ,1,pFile);
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer(	data_, &detId_,				sizeof(detId_));
		data_ = FillBuffer(	data_, &Radius,				sizeof(Radius));
		data_ = FillBuffer( data_, &PosX,				sizeof(PosX));
		data_ = FillBuffer( data_, &PosY,				sizeof(PosY));
		data_ = FillBuffer( data_, &PosZ,				sizeof(PosZ));
		data_ = FillBuffer( data_, &DirX,				sizeof(DirX));
		data_ = FillBuffer( data_, &DirY,				sizeof(DirY));
		data_ = FillBuffer( data_, &DirZ,				sizeof(DirZ));
		data_ = FillBuffer( data_, &NPhi,				sizeof(NPhi));
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);

		glLineWidth(style_->thickness_);
        if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		glLoadName( detId_ );
	
		double dphi  = 6.283185307179586476925286766559 /NPhi;
		double D     = sqrt(DirX*DirX+DirY*DirY+DirZ*DirZ);
				
		glPushMatrix();
		glTranslatef(PosX,PosY,PosZ);
		glRotatef(57.29*acos(DirX/D),1,0,0);
		glRotatef(57.29*acos(DirY/D),0,1,0);
		glRotatef(57.29*acos(DirZ/D),0,0,1);
		glBegin (GL_TRIANGLES); 			
			for(double phi=0; phi<=6.2831;phi+=dphi){			
				glVertex3f (Radius*cos(phi)		 ,Radius*sin(phi)		,0);	// FIRST FACE
				glVertex3f (Radius*cos(phi+dphi) ,Radius*sin(phi+dphi)	,0);
				glVertex3f (0					 ,0						,0);

				glVertex3f (Radius*cos(phi+dphi) ,Radius*sin(phi+dphi)	,0);	// SECOND FACE
				glVertex3f (Radius*cos(phi)		 ,Radius*sin(phi)		,0);
				glVertex3f (0					 ,0						,0);
			}
		glEnd(); 
		glPopMatrix();
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects*) frogObjects;			
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL && mother_->style_ !=NULL){		style_ = new FROG_Objects_Style(mother_->style_);
			}else{												style_ = new FROG_Objects_Style();	}	

			frogObjects_->frogCard_->GetColor( style_->color_     ,"Id_%i_Color"		,detId_);
			frogObjects_->frogCard_->GetFloat(&style_->thickness_ ,"Id_%i_Thickness"	,detId_);
		}
	}
};
#endif
